@@ -1,5 +1,6 @@
 ## creates an object of class "cophylo"
-## written by Liam J. Revell 2015, 2016, 2017
+## written by Liam J. Revell 2015, 2016, 2017, 2019
+
 cophylo<-function(tr1,tr2,assoc=NULL,rotate=TRUE,...){
 	if(!inherits(tr1,"phylo")||!inherits(tr2,"phylo")) 
 		stop("tr1 & tr2 should be objects of class \"phylo\".")
@@ -111,6 +112,7 @@ cophylo<-function(tr1,tr2,assoc=NULL,rotate=TRUE,...){
 
 ## called internally by plot.cophylo to plot a phylogram
 ## written by Liam J. Revell
+
 phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,...){
 	if(hasArg(pts)) pts<-list(...)$pts
 	else pts<-TRUE
@@ -122,6 +124,7 @@ phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,.
 	else tip.lty<-"dotted"
 	if(hasArg(tip.len)) tip.len<-list(...)$tip.len
 	else tip.len<-0.1
+	if(pts==TRUE&&tip.len==0) tip.len<-0.1
 	d<-if(direction=="rightwards") 1 else -1
 	## sub "_" for " "
 	tree$tip.label<-gsub("_"," ",tree$tip.label)
@@ -130,9 +133,12 @@ phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,.
 	## rescale tree so it fits in one half of the plot
 	## with enough space for labels
 	if(ftype=="off") fsize<-0
-	sh<-fsize*strwidth(tree$tip.label)
-	tree$edge.length<-tree$edge.length/max(nodeHeights(tree))*(part-max(sh))
 	n<-Ntip(tree)
+	sh<-fsize*strwidth(tree$tip.label)
+	H<-nodeHeights(tree)
+	th<-sapply(1:n,function(i,x,e) x[which(e==i)],x=H[,2],
+		e=tree$edge[,2])+tip.len*max(H)
+	tree$edge.length<-tree$edge.length/max(th/(part-sh))
 	## reorder cladewise to assign tip positions
 	cw<-reorder(tree,"cladewise")
 	y<-vector(length=n+cw$Nnode)
@@ -164,20 +170,21 @@ phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,.
 		segments(x0=d*xx[1:(length(xx)-1)],y0=yy[1:(length(yy)-1)],
 			x1=d*xx[2:length(xx)],y1=yy[2:length(yy)],lwd=lwd,lend=2,col=edge.col[ee])
 	}
+	h<-part-0.5-tip.len*(max(X)-min(X))-fsize*strwidth(tree$tip.label)
 	## plot links to tips
-	h<-max(X)+tip.len*(max(X)-min(X))+max(fsize*strwidth(tree$tip.label))-
-		fsize*strwidth(tree$tip.label)
 	for(i in 1:n){ 
-		lines(d*c(X[which(cw$edge[,2]==i),2],h[i]),rep(y[i],2),lwd=tip.lwd,lty=tip.lty)
+		lines(d*c(X[which(cw$edge[,2]==i),2],h[i]+tip.len*(max(X)-min(X))),rep(y[i],2),
+			lwd=tip.lwd,lty=tip.lty)
 		if(pts) points(d*X[which(cw$edge[,2]==i),2],y[i],pch=16,cex=pts*0.7*sqrt(lwd))
-	}
+	}	
 	## plot tip labels
 	font<-which(c("off","reg","b","i","bi")==ftype)-1
 	if(font>0){
-		for(i in 1:n) text(d*max(h+fsize*strwidth(tree$tip.label)),y[i],
+		for(i in 1:n) TEXTBOX(d*(h[i]+fsize*strwidth(tree$tip.label[i])+
+			tip.len*(max(X)-min(X))),y[i],
 			tree$tip.label[i], pos=if(d<0) 4 else 2,offset=0,
 			cex=fsize,font=font)
-	}
+	}	
 	PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
 		show.tip.label=if(ftype!="off") TRUE else FALSE,show.node.label=FALSE,
 		font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=0,
@@ -187,11 +194,23 @@ phylogram<-function(tree,part=1,direction="rightwards",fsize=1,ftype="i",lwd=1,.
 		function(x,y,z) y[match(x,z)],y=X,z=cw$edge),yy=y)
 	assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 	## return rightmost or leftmost edge of tip labels
-	invisible(d*max(h+fsize*strwidth(tree$tip.label)))
+	invisible(d*max(h+fsize*strwidth(tree$tip.label)+tip.len*(max(X)-min(X))))
 }
 
+## internally used function
+
+TEXTBOX<-function(x,y,label,pos,offset,cex,font){
+	rect(x,y-0.5*strheight(label,cex=cex,font=font),x+if(pos==4) strwidth(label,
+		cex=cex,font=font) else -strwidth(label,cex=cex,font=font),
+		y+0.5*strheight(label,cex=cex,font=font),border=FALSE,
+		col=if(par()$bg%in%c("white","transparent")) "white" else par()$bg)
+	text(x=x,y=y,label=label,pos=pos,offset=offset,cex=cex,font=font)
+}
+	
+
 ## plot links between tip taxa according to assoc
-## written by Liam J. Revell 2015, 2016
+## written by Liam J. Revell 2015, 2016, 2019
+
 makelinks<-function(obj,x,link.type="curved",link.lwd=1,link.col="black",
 	link.lty="dashed"){
 	if(length(link.lwd)==1) link.lwd<-rep(link.lwd,nrow(obj$assoc))
@@ -200,15 +219,19 @@ makelinks<-function(obj,x,link.type="curved",link.lwd=1,link.col="black",
 	for(i in 1:nrow(obj$assoc)){
 		ii<-which(obj$trees[[1]]$tip.label==obj$assoc[i,1])
 		jj<-which(obj$trees[[2]]$tip.label==obj$assoc[i,2])
-		y<-c((ii-1)/(Ntip(obj$trees[[1]])-1),(jj-1)/(Ntip(obj$trees[[2]])-1))
-		if(link.type=="straight") lines(x,y,lty=link.lty[i],
-			lwd=link.lwd[i],col=link.col[i])
-		else if(link.type=="curved") drawCurve(x,y,lty=link.lty[i],
-			lwd=link.lwd[i],col=link.col[i])
+		for(j in 1:length(ii)) for(k in 1:length(jj)){
+			y<-c((ii[j]-1)/(Ntip(obj$trees[[1]])-1),
+				(jj[k]-1)/(Ntip(obj$trees[[2]])-1))
+			if(link.type=="straight") lines(x,y,lty=link.lty[i],
+				lwd=link.lwd[i],col=link.col[i])
+			else if(link.type=="curved") drawCurve(x,y,lty=link.lty[i],
+				lwd=link.lwd[i],col=link.col[i])
+		}
 	}
 }
 
 ## plot method for class "multiCophylo"
+
 plot.multiCophylo<-function(x,...){
 	par(ask=TRUE)
 	for(i in 1:length(x)) plot.cophylo(x[[i]],...)
@@ -216,6 +239,7 @@ plot.multiCophylo<-function(x,...){
 
 ## plot an object of class "cophylo"
 ## written by Liam J. Revell 2015, 2016, 2017
+
 plot.cophylo<-function(x,...){
 	plot.new()
 	if(hasArg(mar)) mar<-list(...)$mar
@@ -266,6 +290,7 @@ plot.cophylo<-function(x,...){
 
 ## add scale bar
 ## written by Liam J. Revell 2015
+
 add.scalebar<-function(obj,scale.bar,fsize){
 	if(scale.bar[1]>0){
 		s1<-(0.4-max(fsize*strwidth(obj$trees[[1]]$tip.label)))/max(nodeHeights(obj$trees[[1]]))
@@ -285,6 +310,7 @@ add.scalebar<-function(obj,scale.bar,fsize){
 
 ## print an object of class "cophylo"
 ## written by Liam J. Revell 2015
+
 print.cophylo<-function(x, ...){
     cat("Object of class \"cophylo\" containing:\n\n")
     cat("(1) 2 (possibly rotated) phylogenetic trees in an object of class \"multiPhylo\".\n\n")
@@ -292,10 +318,12 @@ print.cophylo<-function(x, ...){
 }
 
 ## print method for "multiCophylo" object
+
 print.multiCophylo<-function(x, ...)
 	cat("Object of class \"multiCophylo\" containg",length(x),"objects of class \"cophylo\".\n\n")
 
 ## written by Liam J. Revell 2015, 2016
+
 tipRotate<-function(tree,x,...){
 	if(hasArg(fn)) fn<-list(...)$fn
 	else fn<-function(x) x^2
@@ -308,7 +336,7 @@ tipRotate<-function(tree,x,...){
 	else max.exhaustive<-20
 	if(hasArg(rotate.multi)) rotate.multi<-list(...)$rotate.multi
 	else rotate.multi<-FALSE
-	if(rotate.multi) rotate.multi<-!is.binary.tree(tree)
+	if(rotate.multi) rotate.multi<-!is.binary(tree)
 	if(hasArg(anim.cophylo)) anim.cophylo<-list(...)$anim.cophylo
 	else anim.cophylo<-FALSE
 	if(anim.cophylo){
@@ -459,7 +487,7 @@ summary.cophylo<-function(object,...){
 		Ntip(object$trees[[2]]),"species.\n\n"))
 	cat("Association (assoc) table as follows:\n\n")
 	maxl<-max(sapply(strsplit(object$assoc[,1],""),length))
-	cat(paste("\tleft:",paste(rep(" ",maxl-5),collapse=""),
+	cat(paste("\tleft:",paste(rep(" ",max(0,maxl-5)),collapse=""),
 		"\t----\tright:\n",sep=""))
 	nulo<-apply(object$assoc,1,function(x,maxl) cat(paste("\t",x[1],
 		paste(rep(" ",maxl-length(strsplit(x[1],split="")[[1]])),
