@@ -1,5 +1,5 @@
 ## function for conditional likelihoods at nodes
-## written by Liam J. Revell 2015, 2016
+## written by Liam J. Revell 2015, 2016, 2019, 2020
 ## with input from (& structural similarity to) function ace by E. Paradis et al. 2013
 
 fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
@@ -11,6 +11,10 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	else opt.method<-"nlminb"
 	if(hasArg(min.q)) min.q<-list(...)$min.q
 	else min.q<-1e-12
+	if(hasArg(max.q)) max.q<-list(...)$max.q
+	else max.q<-max(nodeHeights(tree))*100
+	if(hasArg(logscale)) logscale<-list(...)$logscale
+	else logscale<-FALSE
 	N<-Ntip(tree)
 	M<-tree$Nnode
 	if(is.matrix(x)){
@@ -25,14 +29,24 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	}
 	if(hasArg(pi)) pi<-list(...)$pi
 	else pi<-"equal"
-	if(pi[1]=="equal") pi<-setNames(rep(1/m,m),states)
-	else if(pi[1]=="estimated"){ 
-		pi<-if(!is.null(fixedQ)) statdist(fixedQ) else statdist(summary(fitMk(tree,x,model),quiet=TRUE)$Q)
-		cat("Using pi estimated from the stationary distribution of Q assuming a flat prior.\npi =\n")
+	if(is.numeric(pi)) root.prior<-"given"
+	if(pi[1]=="equal"){ 
+		pi<-setNames(rep(1/m,m),states)
+		root.prior<-"flat"
+	} else if(pi[1]=="estimated"){ 
+		pi<-if(!is.null(fixedQ)) statdist(fixedQ) else 
+			statdist(summary(fitMk(tree,x,model),quiet=TRUE)$Q)
+		cat(paste("Using pi estimated from the stationary",
+			"distribution of Q assuming a flat prior.\npi =\n"))
 		print(round(pi,6))
 		cat("\n")
-	}	
-	else pi<-pi/sum(pi)
+		root.prior<-"stationary"
+	} else if(pi[1]=="fitzjohn") root.prior<-"nuisance"
+	if(is.numeric(pi)){ 
+		pi<-pi/sum(pi)
+		if(is.null(names(pi))) pi<-setNames(pi,states)
+		pi<-pi[states]
+	} 
 	if(is.null(fixedQ)){
 		if(is.character(model)){
 			rate<-matrix(NA,m,m)
@@ -70,11 +84,12 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 	rate[rate==0]<-k+1
 	liks<-rbind(x,matrix(0,M,m,dimnames=list(1:M+N,states)))
 	pw<-reorder(tree,"pruningwise")
-	lik<-function(pp,output.liks=FALSE,pi){
-		if(any(is.nan(pp))||any(is.infinite(pp))) return(1e50)
+	lik<-function(Q,output.liks=FALSE,pi,...){
+		if(hasArg(output.pi)) output.pi<-list(...)$output.pi
+		else output.pi<-FALSE
+		if(is.Qmatrix(Q)) Q<-unclass(Q)
+		if(any(is.nan(Q))||any(is.infinite(Q))) return(1e50)
 		comp<-vector(length=N+M,mode="numeric")
-		Q[]<-c(pp,0)[rate]
-		diag(Q)<--rowSums(Q)
 		parents<-unique(pw$edge[,1])
 		root<-min(parents)
 		for(i in 1:length(parents)){
@@ -83,43 +98,89 @@ fitMk<-function(tree,x,model="SYM",fixedQ=NULL,...){
 			desc<-pw$edge[ii,2]
 			el<-pw$edge.length[ii]
 			v<-vector(length=length(desc),mode="list")
-			for(j in 1:length(v))
+			for(j in 1:length(v)){
 				v[[j]]<-EXPM(Q*el[j])%*%liks[desc[j],]
-			vv<-if(anc==root) Reduce('*',v)[,1]*pi else Reduce('*',v)[,1]
+			}
+			if(anc==root){
+				if(is.numeric(pi)) vv<-Reduce('*',v)[,1]*pi
+				else if(pi[1]=="fitzjohn"){
+					D<-Reduce('*',v)[,1]
+					pi<-D/sum(D)
+					vv<-D*D/sum(D)
+				}
+			} else vv<-Reduce('*',v)[,1]
+			## vv<-if(anc==root) Reduce('*',v)[,1]*pi else Reduce('*',v)[,1]
 			comp[anc]<-sum(vv)
 			liks[anc,]<-vv/comp[anc]
 		}
-		if(output.liks)return(liks[1:M+N,,drop=FALSE]) 
-		logL<--sum(log(comp[1:M+N]))
-		return(if(is.na(logL)) Inf else logL)
+		if(output.liks) return(liks[1:M+N,,drop=FALSE])
+		else if(output.pi) return(pi)
+		else {
+			logL<--sum(log(comp[1:M+N]))
+			if(is.na(logL)) logL<-Inf
+			return(logL)
+		}
 	}
 	if(is.null(fixedQ)){
 		if(length(q.init)!=k) q.init<-rep(q.init[1],k)
-		if(opt.method=="optim")
-			fit<-optim(q.init,function(p) lik(p,pi=pi),method="L-BFGS-B",lower=rep(min.q,k))
-		else	
-			fit<-nlminb(q.init,function(p) lik(p,pi=pi),lower=rep(0,k),upper=rep(1e50,k))
+		q.init<-if(logscale) log(q.init) else q.init
+		if(opt.method=="optim"){
+			fit<-if(logscale) 
+				optim(q.init,function(p) lik(makeQ(m,exp(p),index.matrix),pi=pi),
+					method="L-BFGS-B",lower=rep(log(min.q),k),upper=rep(log(max.q),k)) else
+				optim(q.init,function(p) lik(makeQ(m,p,index.matrix),pi=pi),
+					method="L-BFGS-B",lower=rep(min.q,k),upper=rep(max.q,k))
+		} else if(opt.method=="none"){
+			fit<-list(objective=lik(makeQ(m,q.init,index.matrix),pi=pi),
+				par=q.init)
+		} else {
+			fit<-if(logscale)
+				nlminb(q.init,function(p) lik(makeQ(m,exp(p),index.matrix),pi=pi),
+					lower=rep(log(min.q),k),upper=rep(log(max.q),k))
+				else nlminb(q.init,function(p) lik(makeQ(m,p,index.matrix),
+					pi=pi),lower=rep(0,k),upper=rep(max.q,k))
+		}
+		if(logscale) fit$par<-exp(fit$par)
+		if(pi[1]=="fitzjohn") pi<-setNames(
+			lik(makeQ(m,fit$par,index.matrix),FALSE,pi=pi,output.pi=TRUE),
+			states)
 		obj<-list(logLik=
 			if(opt.method=="optim") -fit$value else -fit$objective,
 			rates=fit$par,
 			index.matrix=index.matrix,
 			states=states,
 			pi=pi,
-			method=opt.method)
-		if(output.liks) obj$lik.anc<-lik(obj$rates,TRUE,pi=pi)
+			method=opt.method,
+			root.prior=root.prior)
+		if(output.liks) obj$lik.anc<-lik(makeQ(m,obj$rates,index.matrix),TRUE,
+			pi=pi)
 	} else {
-		fit<-lik(Q[sapply(1:k,function(x,y) which(x==y),index.matrix)],pi=pi)
+		fit<-lik(Q,pi=pi)
+		if(pi[1]=="fitzjohn") pi<-setNames(lik(Q,FALSE,pi=pi,output.pi=TRUE),states)
 		obj<-list(logLik=-fit,
 			rates=Q[sapply(1:k,function(x,y) which(x==y),index.matrix)],
 			index.matrix=index.matrix,
 			states=states,
+			pi=pi,
+			root.prior=root.prior)
+		if(output.liks) obj$lik.anc<-lik(makeQ(m,obj$rates,index.matrix),TRUE,
 			pi=pi)
-		if(output.liks) obj$lik.anc<-lik(obj$rates,TRUE,pi=pi)
 	}
+	lik.f<-function(q) -lik(q,output.liks=FALSE,
+		pi=if(root.prior=="nuisance") "fitzjohn" else pi)
+	obj$lik<-lik.f
 	class(obj)<-"fitMk"
 	return(obj)
 }
 
+makeQ<-function(m,q,index.matrix){
+	Q<-matrix(0,m,m)
+	Q[]<-c(0,q)[index.matrix+1]
+	diag(Q)<-0
+	diag(Q)<--rowSums(Q)
+	Q
+}
+
 ## print method for objects of class "fitMk"
 print.fitMk<-function(x,digits=6,...){
 	cat("Object of class \"fitMk\".\n\n")
@@ -131,9 +192,12 @@ print.fitMk<-function(x,digits=6,...){
 	colnames(Q)<-rownames(Q)<-x$states
 	print(round(Q,digits))
 	cat("\nFitted (or set) value of pi:\n")
-	print(x$pi)
+	print(round(x$pi,digits))
+	cat(paste("due to treating the root prior as (a) ",x$root.prior,".\n",
+		sep=""))
 	cat(paste("\nLog-likelihood:",round(x$logLik,digits),"\n"))
-	cat(paste("\nOptimization method used was \"",x$method,"\"\n\n",sep=""))
+	cat(paste("\nOptimization method used was \"",x$method,"\"\n\n",
+		sep=""))
 }
 
 ## summary method for objects of class "fitMk"
@@ -160,14 +224,39 @@ logLik.fitMk<-function(object,...){
 	lik
 }
 
-## AIC method
-AIC.fitMk<-function(object,...,k=2){
-	np<-length(object$rates)
-	-2*logLik(object)+np*k
-}
-	
 ## S3 plot method for objects of class "fitMk"
 plot.fitMk<-function(x,...){
+	Q<-as.Qmatrix(x)
+	plot(Q,...)
+}
+
+## S3 plot method for "gfit" object from geiger::fitDiscrete
+plot.gfit<-function(x,...){
+	if("mkn"%in%class(x$lik)==FALSE){
+		stop("Sorry. No plot method presently available for objects of this type.")
+		object<-NULL
+	} else {
+		chk<-.check.pkg("geiger")
+		if(chk) object<-plot(as.Qmatrix(x),...)
+		else {
+			obj<-list()
+			QQ<-.Qmatrix.from.gfit(x)
+			obj$states<-colnames(QQ)
+			m<-length(obj$states)
+			obj$index.matrix<-matrix(NA,m,m)
+			k<-m*(m-1)
+			obj$index.matrix[col(obj$index.matrix)!=row(obj$index.matrix)]<-1:k
+			obj$rates<-QQ[sapply(1:k,function(x,y) which(x==y),obj$index.matrix)]
+			class(obj)<-"fitMk"
+			object<-plot(obj,...)
+		}
+	}
+	invisible(object)
+}
+	
+## S3 method for "Qmatrix" object class
+plot.Qmatrix<-function(x,...){
+	Q<-unclass(x)
 	if(hasArg(signif)) signif<-list(...)$signif
 	else signif<-3
 	if(hasArg(main)) main<-list(...)$main
@@ -186,21 +275,34 @@ plot.fitMk<-function(x,...){
 	else mar<-c(1.1,1.1,3.1,1.1)
 	if(hasArg(lwd)) lwd<-list(...)$lwd
 	else lwd<-1
-	Q<-matrix(NA,length(x$states),length(x$states))
-    	Q[]<-c(0,x$rates)[x$index.matrix+1]
-	diag(Q)<-0
-	spacer<-0.1
+	if(hasArg(umbral)) umbral<-list(...)$umbral	
+	else umbral<-FALSE
+	if(hasArg(ncat)) ncat<-list(...)$ncat
+	else ncat<-NULL
+	if(hasArg(spacer)) spacer<-list(...)$spacer
+	else spacer<-0.1
 	plot.new()
 	par(mar=mar)
 	xylim<-c(-1.2,1.2)
 	plot.window(xlim=xylim,ylim=xylim,asp=1)
 	if(!is.null(main)) title(main=main,cex.main=cex.main)
-	nstates<-length(x$states)
-	step<-360/nstates
-	angles<-seq(0,360-step,by=step)/180*pi
-	if(nstates==2) angles<-angles+pi/2
-	v.x<-cos(angles)
-	v.y<-sin(angles)
+	nstates<-nrow(Q)
+	if(!umbral||is.null(ncat)){
+		step<-360/nstates
+		angles<-seq(0,360-step,by=step)/180*pi
+		if(nstates==2) angles<-angles+pi/2
+		v.x<-cos(angles)
+		v.y<-sin(angles)
+	} else {
+		v.x<-v.y<-vector()
+		for(i in 1:length(ncat)){
+			Q<-Q[sort(rownames(Q)),sort(colnames(Q))]
+			xp<--1+2*(i-1)/(length(ncat)-1)
+			v.x<-c(v.x,rep(xp,ncat[i]))
+			yp<-seq(1,-1,length.out=max(ncat))[1:ncat[i]]
+			v.y<-c(v.y,yp)
+		}
+	}	
 	for(i in 1:nstates) for(j in 1:nstates)
 		if(if(!isSymmetric(Q)) i!=j else i>j){
 			dx<-v.x[j]-v.x[i]
@@ -233,26 +335,10 @@ plot.fitMk<-function(x,...){
 					code=if(isSymmetric(Q)) 3 else 2,lwd=lwd)
 			}
 		}
-	text(v.x,v.y,x$states,cex=cex.traits,
+	text(v.x,v.y,rownames(Q),cex=cex.traits,
 		col=make.transparent("black",0.7))
-}
-
-## S3 plot method for objects resulting from fitDiscrete
-plot.gfit<-function(x,...){
-	if("mkn"%in%class(x$lik)==FALSE){
-		stop("Sorry. No plot method presently available for objects of this type.")
-	} else {
-		obj<-list()
-		QQ<-.Qmatrix.from.gfit(x)
-		obj$states<-colnames(QQ)
-		m<-length(obj$states)
-		obj$index.matrix<-matrix(NA,m,m)
-		k<-m*(m-1)
-		obj$index.matrix[col(obj$index.matrix)!=row(obj$index.matrix)]<-1:k
-		obj$rates<-QQ[sapply(1:k,function(x,y) which(x==y),obj$index.matrix)]
-		class(obj)<-"fitMk"
-		plot(obj,...)
-	}
+	object<-data.frame(states=rownames(Q),x=v.x,y=v.y)
+	invisible(object)
 }
 
 ## wraps around expm
@@ -324,3 +410,34 @@ sim.Mk<-function(tree,Q,anc=NULL,nsim=1,...){
 	}
 	X
 }
+
+## as.Qmatrix method
+
+as.Qmatrix<-function(x,...){
+	if(identical(class(x),"Qmatrix")) return(x)
+	UseMethod("as.Qmatrix")
+}
+
+as.Qmatrix.default<-function(x, ...){
+	warning(paste(
+		"as.Qmatrix does not know how to handle objects of class ",
+		class(x),"."))
+}
+
+as.Qmatrix.fitMk<-function(x,...){
+	Q<-matrix(NA,length(x$states),length(x$states))
+	Q[]<-c(0,x$rates)[x$index.matrix+1]
+	rownames(Q)<-colnames(Q)<-x$states
+	diag(Q)<-0
+	diag(Q)<--rowSums(Q)
+	class(Q)<-"Qmatrix"
+	Q
+}
+
+print.Qmatrix<-function(x,...){
+	cat("Estimated Q matrix:\n")
+	print(unclass(x),...)
+}
+
+is.Qmatrix<-function(x) "Qmatrix" %in% class(x)
+
